=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2010-03-09 22:42:17 +0000
+++ drizzled/cursor.cc	2010-03-16 21:30:44 +0000
@@ -35,6 +35,7 @@
 #include "drizzled/cost_vect.h"
 #include "drizzled/session.h"
 #include "drizzled/sql_base.h"
+#include "drizzled/transaction_services.h"
 #include "drizzled/replication_services.h"
 #include "drizzled/lock.h"
 #include "drizzled/item/int.h"
@@ -738,12 +739,12 @@
     /** 
      * Trigger post-truncate notification to plugins... 
      *
-     * @todo Make ReplicationServices generic to AfterTriggerServices
+     * @todo Make TransactionServices generic to AfterTriggerServices
      * or similar...
      */
     Session *const session= table->in_use;
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    replication_services.truncateTable(session, table);
+    TransactionServices &transaction_services= TransactionServices::singleton();
+    transaction_services.truncateTable(session, table);
   }
 
   return result;
@@ -1345,10 +1346,11 @@
                                     const unsigned char *before_record,
                                     const unsigned char *after_record)
 {
+  TransactionServices &transaction_services= TransactionServices::singleton();
   ReplicationServices &replication_services= ReplicationServices::singleton();
   Session *const session= table->in_use;
 
-  if (table->s->tmp_table || ! replication_services.isActive())
+  if (table->s->tmp_table || not replication_services.isActive())
     return false;
 
   bool result= false;
@@ -1365,7 +1367,7 @@
      * CREATE TABLE will commit the transaction containing
      * it).
      */
-    result= replication_services.insertRecord(session, table);
+    result= transaction_services.insertRecord(session, table);
     break;
   case SQLCOM_REPLACE:
   case SQLCOM_REPLACE_SELECT:
@@ -1389,20 +1391,20 @@
      */
     if (after_record == NULL)
     {
-      replication_services.deleteRecord(session, table);
+      transaction_services.deleteRecord(session, table);
       /* 
        * We set the "current" statement message to NULL.  This triggers
        * the replication services component to generate a new statement
        * message for the inserted record which will come next.
        */
-      replication_services.finalizeStatement(*session->getStatementMessage(), session);
+      transaction_services.finalizeStatementMessage(*session->getStatementMessage(), session);
     }
     else
     {
       if (before_record == NULL)
-        result= replication_services.insertRecord(session, table);
+        result= transaction_services.insertRecord(session, table);
       else
-        replication_services.updateRecord(session, table, before_record, after_record);
+        transaction_services.updateRecord(session, table, before_record, after_record);
     }
     break;
   case SQLCOM_INSERT:
@@ -1414,17 +1416,17 @@
      * an update.
      */
     if (before_record == NULL)
-      result= replication_services.insertRecord(session, table);
+      result= transaction_services.insertRecord(session, table);
     else
-      replication_services.updateRecord(session, table, before_record, after_record);
+      transaction_services.updateRecord(session, table, before_record, after_record);
     break;
 
   case SQLCOM_UPDATE:
-    replication_services.updateRecord(session, table, before_record, after_record);
+    transaction_services.updateRecord(session, table, before_record, after_record);
     break;
 
   case SQLCOM_DELETE:
-    replication_services.deleteRecord(session, table);
+    transaction_services.deleteRecord(session, table);
     break;
   default:
     break;

=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2010-03-04 08:35:47 +0000
+++ drizzled/db.cc	2010-03-16 21:30:44 +0000
@@ -35,7 +35,7 @@
 #include <drizzled/sql_base.h>
 #include <drizzled/lock.h>
 #include <drizzled/errmsg_print.h>
-#include <drizzled/replication_services.h>
+#include <drizzled/transaction_services.h>
 #include <drizzled/message/schema.pb.h>
 #include "drizzled/sql_table.h"
 #include "drizzled/plugin/storage_engine.h"
@@ -81,7 +81,7 @@
 
 bool mysql_create_db(Session *session, const message::Schema &schema_message, const bool is_if_not_exists)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
+  TransactionServices &transaction_services= TransactionServices::singleton();
   bool error= false;
 
   /*
@@ -130,7 +130,7 @@
   }
   else // Created !
   {
-    replication_services.createSchema(session, schema_message);
+    transaction_services.createSchema(session, schema_message);
     session->my_ok(1);
   }
 
@@ -145,7 +145,7 @@
 
 bool mysql_alter_db(Session *session, const message::Schema &schema_message)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
+  TransactionServices &transaction_services= TransactionServices::singleton();
 
   /*
     Do not alter database if another thread is holding read lock.
@@ -175,7 +175,7 @@
 
   if (success)
   {
-    replication_services.rawStatement(session, session->getQueryString());
+    transaction_services.rawStatement(session, session->getQueryString());
     session->my_ok(1);
   }
   else
@@ -275,8 +275,8 @@
   {
     assert(! session->query.empty());
 
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    replication_services.dropSchema(session, schema_name);
+    TransactionServices &transaction_services= TransactionServices::singleton();
+    transaction_services.dropSchema(session, schema_name);
     session->clear_error();
     session->server_status|= SERVER_STATUS_DB_DROPPED;
     session->my_ok((uint32_t) deleted);
@@ -293,7 +293,7 @@
     query_end= query + MAX_DROP_TABLE_Q_LEN;
     db_len= schema_name.length();
 
-    ReplicationServices &replication_services= ReplicationServices::singleton();
+    TransactionServices &transaction_services= TransactionServices::singleton();
     for (plugin::TableNameList::iterator it= dropped_tables.begin();
          it != dropped_tables.end();
          it++)
@@ -305,7 +305,7 @@
       if (query_pos + tbl_name_len + 1 >= query_end)
       {
         /* These DDL methods and logging protected with LOCK_create_db */
-        replication_services.rawStatement(session, query);
+        transaction_services.rawStatement(session, query);
         query_pos= query_data_start;
       }
 
@@ -318,7 +318,7 @@
     if (query_pos != query_data_start)
     {
       /* These DDL methods and logging protected with LOCK_create_db */
-      replication_services.rawStatement(session, query);
+      transaction_services.rawStatement(session, query);
     }
   }
 

=== modified file 'drizzled/replication_services.cc'
--- drizzled/replication_services.cc	2010-03-15 23:02:06 +0000
+++ drizzled/replication_services.cc	2010-03-16 21:30:44 +0000
@@ -2,10 +2,11 @@
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
  *  Copyright (C) 2008-2009 Sun Microsystems
+ *  Copyright (c) 2009-2010 Jay Pipes <jaypipes@gmail.com>
  *
  *  Authors:
  *
- *    Jay Pipes <joinfu@sun.com>
+ *    Jay Pipes <jaypipes@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,32 +24,15 @@
 
 /**
  * @file Server-side utility which is responsible for managing the 
- * communication between the kernel, replicator plugins, and applier plugins.
- *
- * ReplicationServices is a bridge between modules and the kernel, and its
- * primary function is to take internal events (for instance the start of a 
- * transaction, the changing of a record, or the rollback of a transaction) 
- * and construct GPB Messages that are passed to the registered replicator and
- * applier plugins.
- *
- * The reason for this functionality is to encapsulate all communication
- * between the kernel and the replicator/applier plugins into GPB Messages.
- * Instead of the plugin having to understand the (often fluidly changing)
- * mechanics of the kernel, all the plugin needs to understand is the message
- * format, and GPB messages provide a nice, clear, and versioned format for 
- * these messages.
- *
- * @see /drizzled/message/transaction.proto
- *
- * @todo
- *
- * We really should store the raw bytes in the messages, not the
- * String value of the Field.  But, to do that, the
- * statement_transform library needs first to be updated
- * to include the transformation code to convert raw
- * Drizzle-internal Field byte representation into something
- * plugins can understand.
- */
+ * communication between the kernel and the replication plugins:
+ *
+ * - TransactionReplicator
+ * - TransactionApplier
+ * - Publisher
+ * - Subscriber
+ *
+ * ReplicationServices is a bridge between replication modules and the kernel,
+ * and its primary function is to  */
 
 #include "config.h"
 #include "drizzled/replication_services.h"
@@ -161,707 +145,7 @@
   return is_active;
 }
 
-message::Transaction *ReplicationServices::getActiveTransaction(Session *in_session) const
-{
-  message::Transaction *transaction= in_session->getTransactionMessage();
-
-  if (unlikely(transaction == NULL))
-  {
-    /* 
-     * Allocate and initialize a new transaction message 
-     * for this Session object.  Session is responsible for
-     * deleting transaction message when done with it.
-     */
-    transaction= new (nothrow) message::Transaction();
-    initTransaction(*transaction, in_session);
-    in_session->setTransactionMessage(transaction);
-    return transaction;
-  }
-  else
-    return transaction;
-}
-
-void ReplicationServices::initTransaction(message::Transaction &in_transaction,
-                                          Session *in_session) const
-{
-  message::TransactionContext *trx= in_transaction.mutable_transaction_context();
-  trx->set_server_id(in_session->getServerId());
-  trx->set_transaction_id(in_session->getQueryId());
-  trx->set_start_timestamp(in_session->getCurrentTimestamp());
-}
-
-void ReplicationServices::finalizeTransaction(message::Transaction &in_transaction,
-                                              Session *in_session) const
-{
-  message::TransactionContext *trx= in_transaction.mutable_transaction_context();
-  trx->set_end_timestamp(in_session->getCurrentTimestamp());
-}
-
-void ReplicationServices::cleanupTransaction(message::Transaction *in_transaction,
-                                             Session *in_session) const
-{
-  delete in_transaction;
-  in_session->setStatementMessage(NULL);
-  in_session->setTransactionMessage(NULL);
-}
-
-void ReplicationServices::commitTransaction(Session *in_session)
-{
-  if (! is_active)
-    return;
-
-  /* If there is an active statement message, finalize it */
-  message::Statement *statement= in_session->getStatementMessage();
-
-  if (statement != NULL)
-  {
-    finalizeStatement(*statement, in_session);
-  }
-  else
-    return; /* No data modification occurred inside the transaction */
-  
-  message::Transaction* transaction= getActiveTransaction(in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-}
-
-void ReplicationServices::initStatement(message::Statement &statement,
-                                        message::Statement::Type in_type,
-                                        Session *in_session) const
-{
-  statement.set_type(in_type);
-  statement.set_start_timestamp(in_session->getCurrentTimestamp());
-  /** @TODO Set sql string optionally */
-}
-
-void ReplicationServices::finalizeStatement(message::Statement &statement,
-                                            Session *in_session) const
-{
-  statement.set_end_timestamp(in_session->getCurrentTimestamp());
-  in_session->setStatementMessage(NULL);
-}
-
-void ReplicationServices::rollbackTransaction(Session *in_session)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-
-  /*
-   * OK, so there are two situations that we need to deal with here:
-   *
-   * 1) We receive an instruction to ROLLBACK the current transaction
-   *    and the currently-stored Transaction message is *self-contained*, 
-   *    meaning that no Statement messages in the Transaction message
-   *    contain a message having its segment_id member greater than 1.  If
-   *    no non-segment ID 1 members are found, we can simply clear the
-   *    current Transaction message and remove it from memory.
-   *
-   * 2) If the Transaction message does indeed have a non-end segment, that
-   *    means that a bulk update/delete/insert Transaction message segment
-   *    has previously been sent over the wire to replicators.  In this case, 
-   *    we need to package a Transaction with a Statement message of type
-   *    ROLLBACK to indicate to replicators that previously-transmitted
-   *    messages must be un-applied.
-   */
-  if (unlikely(message::transactionContainsBulkSegment(*transaction)))
-  {
-    /*
-     * Clear the transaction, create a Rollback statement message, 
-     * attach it to the transaction, and push it to replicators.
-     */
-    transaction->Clear();
-    initTransaction(*transaction, in_session);
-
-    message::Statement *statement= transaction->add_statement();
-
-    initStatement(*statement, message::Statement::ROLLBACK, in_session);
-    finalizeStatement(*statement, in_session);
-
-    finalizeTransaction(*transaction, in_session);
-    
-    push(*transaction);
-  }
-  cleanupTransaction(transaction, in_session);
-}
-
-message::Statement &ReplicationServices::getInsertStatement(Session *in_session,
-                                                                 Table *in_table) const
-{
-  message::Statement *statement= in_session->getStatementMessage();
-  /*
-   * We check to see if the current Statement message is of type INSERT.
-   * If it is not, we finalize the current Statement and ensure a new
-   * InsertStatement is created.
-   */
-  if (statement != NULL &&
-      statement->type() != message::Statement::INSERT)
-  {
-    finalizeStatement(*statement, in_session);
-    statement= in_session->getStatementMessage();
-  }
-
-  if (statement == NULL)
-  {
-    message::Transaction *transaction= getActiveTransaction(in_session);
-    /* 
-     * Transaction message initialized and set, but no statement created
-     * yet.  We construct one and initialize it, here, then return the
-     * message after attaching the new Statement message pointer to the 
-     * Session for easy retrieval later...
-     */
-    statement= transaction->add_statement();
-    setInsertHeader(*statement, in_session, in_table);
-    in_session->setStatementMessage(statement);
-  }
-  return *statement;
-}
-
-void ReplicationServices::setInsertHeader(message::Statement &statement,
-                                          Session *in_session,
-                                          Table *in_table) const
-{
-  initStatement(statement, message::Statement::INSERT, in_session);
-
-  /* 
-   * Now we construct the specialized InsertHeader message inside
-   * the generalized message::Statement container...
-   */
-  /* Set up the insert header */
-  message::InsertHeader *header= statement.mutable_insert_header();
-  message::TableMetadata *table_metadata= header->mutable_table_metadata();
-
-  const char *schema_name= in_table->getShare()->db.str;
-  const char *table_name= in_table->getShare()->table_name.str;
-
-  table_metadata->set_schema_name(schema_name);
-  table_metadata->set_table_name(table_name);
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-
-  message::FieldMetadata *field_metadata;
-
-  /* We will read all the table's fields... */
-  in_table->setReadSet();
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    field_metadata= header->add_field_metadata();
-    field_metadata->set_name(current_field->field_name);
-    field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
-  }
-}
-
-bool ReplicationServices::insertRecord(Session *in_session, Table *in_table)
-{
-  if (! is_active)
-    return false;
-  /**
-   * We do this check here because we don't want to even create a 
-   * statement if there isn't a primary key on the table...
-   *
-   * @todo
-   *
-   * Multi-column primary keys are handled how exactly?
-   */
-  if (in_table->s->primary_key == MAX_KEY)
-  {
-    my_error(ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE, MYF(0));
-    return true;
-  }
-
-  message::Statement &statement= getInsertStatement(in_session, in_table);
-
-  message::InsertData *data= statement.mutable_insert_data();
-  data->set_segment_id(1);
-  data->set_end_segment(true);
-  message::InsertRecord *record= data->add_record();
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-
-  String *string_value= new (in_session->mem_root) String(ReplicationServices::DEFAULT_RECORD_SIZE);
-  string_value->set_charset(system_charset_info);
-
-  /* We will read all the table's fields... */
-  in_table->setReadSet();
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    string_value= current_field->val_str(string_value);
-    record->add_insert_value(string_value->c_ptr(), string_value->length());
-    string_value->free();
-  }
-  return false;
-}
-
-message::Statement &ReplicationServices::getUpdateStatement(Session *in_session,
-                                                            Table *in_table,
-                                                            const unsigned char *old_record, 
-                                                            const unsigned char *new_record) const
-{
-  message::Statement *statement= in_session->getStatementMessage();
-  /*
-   * We check to see if the current Statement message is of type UPDATE.
-   * If it is not, we finalize the current Statement and ensure a new
-   * UpdateStatement is created.
-   */
-  if (statement != NULL &&
-      statement->type() != message::Statement::UPDATE)
-  {
-    finalizeStatement(*statement, in_session);
-    statement= in_session->getStatementMessage();
-  }
-
-  if (statement == NULL)
-  {
-    message::Transaction *transaction= getActiveTransaction(in_session);
-    /* 
-     * Transaction message initialized and set, but no statement created
-     * yet.  We construct one and initialize it, here, then return the
-     * message after attaching the new Statement message pointer to the 
-     * Session for easy retrieval later...
-     */
-    statement= transaction->add_statement();
-    setUpdateHeader(*statement, in_session, in_table, old_record, new_record);
-    in_session->setStatementMessage(statement);
-  }
-  return *statement;
-}
-
-void ReplicationServices::setUpdateHeader(message::Statement &statement,
-                                          Session *in_session,
-                                          Table *in_table,
-                                          const unsigned char *old_record, 
-                                          const unsigned char *new_record) const
-{
-  initStatement(statement, message::Statement::UPDATE, in_session);
-
-  /* 
-   * Now we construct the specialized UpdateHeader message inside
-   * the generalized message::Statement container...
-   */
-  /* Set up the update header */
-  message::UpdateHeader *header= statement.mutable_update_header();
-  message::TableMetadata *table_metadata= header->mutable_table_metadata();
-
-  const char *schema_name= in_table->getShare()->db.str;
-  const char *table_name= in_table->getShare()->table_name.str;
-
-  table_metadata->set_schema_name(schema_name);
-  table_metadata->set_table_name(table_name);
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-
-  message::FieldMetadata *field_metadata;
-
-  /* We will read all the table's fields... */
-  in_table->setReadSet();
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    /*
-     * We add the "key field metadata" -- i.e. the fields which is
-     * the primary key for the table.
-     */
-    if (in_table->s->fieldInPrimaryKey(current_field))
-    {
-      field_metadata= header->add_key_field_metadata();
-      field_metadata->set_name(current_field->field_name);
-      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
-    }
-
-    /*
-     * The below really should be moved into the Field API and Record API.  But for now
-     * we do this crazy pointer fiddling to figure out if the current field
-     * has been updated in the supplied record raw byte pointers.
-     */
-    const unsigned char *old_ptr= (const unsigned char *) old_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
-    const unsigned char *new_ptr= (const unsigned char *) new_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
-
-    uint32_t field_length= current_field->pack_length(); /** @TODO This isn't always correct...check varchar diffs. */
-
-    if (memcmp(old_ptr, new_ptr, field_length) != 0)
-    {
-      /* Field is changed from old to new */
-      field_metadata= header->add_set_field_metadata();
-      field_metadata->set_name(current_field->field_name);
-      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
-    }
-  }
-}
-void ReplicationServices::updateRecord(Session *in_session,
-                                       Table *in_table, 
-                                       const unsigned char *old_record, 
-                                       const unsigned char *new_record)
-{
-  if (! is_active)
-    return;
-
-  message::Statement &statement= getUpdateStatement(in_session, in_table, old_record, new_record);
-
-  message::UpdateData *data= statement.mutable_update_data();
-  data->set_segment_id(1);
-  data->set_end_segment(true);
-  message::UpdateRecord *record= data->add_record();
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-  String *string_value= new (in_session->mem_root) String(ReplicationServices::DEFAULT_RECORD_SIZE);
-  string_value->set_charset(system_charset_info);
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    /*
-     * Here, we add the SET field values.  We used to do this in the setUpdateHeader() method, 
-     * but then realized that an UPDATE statement could potentially have different values for
-     * the SET field.  For instance, imagine this SQL scenario:
-     *
-     * CREATE TABLE t1 (id INT NOT NULL PRIMARY KEY, count INT NOT NULL);
-     * INSERT INTO t1 (id, counter) VALUES (1,1),(2,2),(3,3);
-     * UPDATE t1 SET counter = counter + 1 WHERE id IN (1,2);
-     *
-     * We will generate two UpdateRecord messages with different set_value byte arrays.
-     *
-     * The below really should be moved into the Field API and Record API.  But for now
-     * we do this crazy pointer fiddling to figure out if the current field
-     * has been updated in the supplied record raw byte pointers.
-     */
-    const unsigned char *old_ptr= (const unsigned char *) old_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
-    const unsigned char *new_ptr= (const unsigned char *) new_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
-
-    uint32_t field_length= current_field->pack_length(); /** @TODO This isn't always correct...check varchar diffs. */
-
-    if (memcmp(old_ptr, new_ptr, field_length) != 0)
-    {
-      /* Store the original "read bit" for this field */
-      bool is_read_set= current_field->isReadSet();
-
-      /* We need to mark that we will "read" this field... */
-      in_table->setReadSet(current_field->field_index);
-
-      /* Read the string value of this field's contents */
-      string_value= current_field->val_str(string_value);
-
-      /* 
-       * Reset the read bit after reading field to its original state.  This 
-       * prevents the field from being included in the WHERE clause
-       */
-      current_field->setReadSet(is_read_set);
-
-      record->add_after_value(string_value->c_ptr(), string_value->length());
-      string_value->free();
-    }
-
-    /* 
-     * Add the WHERE clause values now...for now, this means the
-     * primary key field value.  Replication only supports tables
-     * with a primary key.
-     */
-    if (in_table->s->fieldInPrimaryKey(current_field))
-    {
-      /**
-       * To say the below is ugly is an understatement. But it works.
-       * 
-       * @todo Move this crap into a real Record API.
-       */
-      string_value= current_field->val_str(string_value,
-                                           old_record + 
-                                           current_field->offset(const_cast<unsigned char *>(new_record)));
-      record->add_key_value(string_value->c_ptr(), string_value->length());
-      string_value->free();
-    }
-
-  }
-}
-
-message::Statement &ReplicationServices::getDeleteStatement(Session *in_session,
-                                                            Table *in_table) const
-{
-  message::Statement *statement= in_session->getStatementMessage();
-  /*
-   * We check to see if the current Statement message is of type DELETE.
-   * If it is not, we finalize the current Statement and ensure a new
-   * DeleteStatement is created.
-   */
-  if (statement != NULL &&
-      statement->type() != message::Statement::DELETE)
-  {
-    finalizeStatement(*statement, in_session);
-    statement= in_session->getStatementMessage();
-  }
-
-  if (statement == NULL)
-  {
-    message::Transaction *transaction= getActiveTransaction(in_session);
-    /* 
-     * Transaction message initialized and set, but no statement created
-     * yet.  We construct one and initialize it, here, then return the
-     * message after attaching the new Statement message pointer to the 
-     * Session for easy retrieval later...
-     */
-    statement= transaction->add_statement();
-    setDeleteHeader(*statement, in_session, in_table);
-    in_session->setStatementMessage(statement);
-  }
-  return *statement;
-}
-
-void ReplicationServices::setDeleteHeader(message::Statement &statement,
-                                          Session *in_session,
-                                          Table *in_table) const
-{
-  initStatement(statement, message::Statement::DELETE, in_session);
-
-  /* 
-   * Now we construct the specialized DeleteHeader message inside
-   * the generalized message::Statement container...
-   */
-  message::DeleteHeader *header= statement.mutable_delete_header();
-  message::TableMetadata *table_metadata= header->mutable_table_metadata();
-
-  const char *schema_name= in_table->getShare()->db.str;
-  const char *table_name= in_table->getShare()->table_name.str;
-
-  table_metadata->set_schema_name(schema_name);
-  table_metadata->set_table_name(table_name);
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-
-  message::FieldMetadata *field_metadata;
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    /* 
-     * Add the WHERE clause values now...for now, this means the
-     * primary key field value.  Replication only supports tables
-     * with a primary key.
-     */
-    if (in_table->s->fieldInPrimaryKey(current_field))
-    {
-      field_metadata= header->add_key_field_metadata();
-      field_metadata->set_name(current_field->field_name);
-      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
-    }
-  }
-}
-
-void ReplicationServices::deleteRecord(Session *in_session, Table *in_table)
-{
-  if (! is_active)
-    return;
-
-  message::Statement &statement= getDeleteStatement(in_session, in_table);
-
-  message::DeleteData *data= statement.mutable_delete_data();
-  data->set_segment_id(1);
-  data->set_end_segment(true);
-  message::DeleteRecord *record= data->add_record();
-
-  Field *current_field;
-  Field **table_fields= in_table->field;
-  String *string_value= new (in_session->mem_root) String(ReplicationServices::DEFAULT_RECORD_SIZE);
-  string_value->set_charset(system_charset_info);
-
-  while ((current_field= *table_fields++) != NULL) 
-  {
-    /* 
-     * Add the WHERE clause values now...for now, this means the
-     * primary key field value.  Replication only supports tables
-     * with a primary key.
-     */
-    if (in_table->s->fieldInPrimaryKey(current_field))
-    {
-      string_value= current_field->val_str(string_value);
-      record->add_key_value(string_value->c_ptr(), string_value->length());
-      /**
-       * @TODO Store optional old record value in the before data member
-       */
-      string_value->free();
-    }
-  }
-}
-
-void ReplicationServices::createTable(Session *in_session,
-                                      const message::Table &table)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::CREATE_TABLE, in_session);
-
-  /* 
-   * Construct the specialized CreateTableStatement message and attach
-   * it to the generic Statement message
-   */
-  message::CreateTableStatement *create_table_statement= statement->mutable_create_table_statement();
-  message::Table *new_table_message= create_table_statement->mutable_table();
-  *new_table_message= table;
-
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-
-}
-
-void ReplicationServices::createSchema(Session *in_session,
-                                       const message::Schema &schema)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::CREATE_SCHEMA, in_session);
-
-  /* 
-   * Construct the specialized CreateSchemaStatement message and attach
-   * it to the generic Statement message
-   */
-  message::CreateSchemaStatement *create_schema_statement= statement->mutable_create_schema_statement();
-  message::Schema *new_schema_message= create_schema_statement->mutable_schema();
-  *new_schema_message= schema;
-
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-
-}
-
-void ReplicationServices::dropSchema(Session *in_session, const string &schema_name)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::DROP_SCHEMA, in_session);
-
-  /* 
-   * Construct the specialized DropSchemaStatement message and attach
-   * it to the generic Statement message
-   */
-  message::DropSchemaStatement *drop_schema_statement= statement->mutable_drop_schema_statement();
-
-  drop_schema_statement->set_schema_name(schema_name);
-
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-}
-
-void ReplicationServices::dropTable(Session *in_session,
-                                    const string &schema_name,
-                                    const string &table_name,
-                                    bool if_exists)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::DROP_TABLE, in_session);
-
-  /* 
-   * Construct the specialized DropTableStatement message and attach
-   * it to the generic Statement message
-   */
-  message::DropTableStatement *drop_table_statement= statement->mutable_drop_table_statement();
-
-  drop_table_statement->set_if_exists_clause(if_exists);
-
-  message::TableMetadata *table_metadata= drop_table_statement->mutable_table_metadata();
-
-  table_metadata->set_schema_name(schema_name);
-  table_metadata->set_table_name(table_name);
-
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-}
-
-void ReplicationServices::truncateTable(Session *in_session, Table *in_table)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::TRUNCATE_TABLE, in_session);
-
-  /* 
-   * Construct the specialized TruncateTableStatement message and attach
-   * it to the generic Statement message
-   */
-  message::TruncateTableStatement *truncate_statement= statement->mutable_truncate_table_statement();
-  message::TableMetadata *table_metadata= truncate_statement->mutable_table_metadata();
-
-  const char *schema_name= in_table->getShare()->db.str;
-  const char *table_name= in_table->getShare()->table_name.str;
-
-  table_metadata->set_schema_name(schema_name);
-  table_metadata->set_table_name(table_name);
-
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-}
-
-void ReplicationServices::rawStatement(Session *in_session, const string &query)
-{
-  if (! is_active)
-    return;
-  
-  message::Transaction *transaction= getActiveTransaction(in_session);
-  message::Statement *statement= transaction->add_statement();
-
-  initStatement(*statement, message::Statement::RAW_SQL, in_session);
-  statement->set_sql(query);
-  finalizeStatement(*statement, in_session);
-
-  finalizeTransaction(*transaction, in_session);
-  
-  push(*transaction);
-
-  cleanupTransaction(transaction, in_session);
-}
-
-void ReplicationServices::push(message::Transaction &to_push)
+void ReplicationServices::pushTransactionMessage(message::Transaction &to_push)
 {
   vector<plugin::TransactionReplicator *>::iterator repl_iter= replicators.begin();
   vector<plugin::TransactionApplier *>::iterator appl_start_iter, appl_iter;

=== modified file 'drizzled/replication_services.h'
--- drizzled/replication_services.h	2010-03-15 23:02:06 +0000
+++ drizzled/replication_services.h	2010-03-16 21:30:44 +0000
@@ -27,8 +27,6 @@
 
 #include "drizzled/atomics.h"
 
-#include "drizzled/message/transaction.pb.h"
-
 #include <vector>
 
 namespace drizzled
@@ -43,6 +41,10 @@
   class TransactionReplicator;
   class TransactionApplier;
 }
+namespace message
+{
+  class Transaction;
+}
 
 /**
  * This is a class which manages transforming internal 
@@ -52,7 +54,6 @@
 class ReplicationServices
 {
 public:
-  static const size_t DEFAULT_RECORD_SIZE= 100;
   typedef uint64_t GlobalTransactionId;
   /**
    * Types of messages that can go in the transaction
@@ -90,126 +91,14 @@
    * This method properly sets the is_active member variable.
    */
   void evaluateActivePlugins();
-  /**
-   * Helper method which returns the active Transaction message
-   * for the supplied Session.  If one is not found, a new Transaction
-   * message is allocated, initialized, and returned.
-   *
-   * @param The session processing the transaction
-   */
-  message::Transaction *getActiveTransaction(Session *in_session) const;
-  /** 
-   * Helper method which attaches a transaction context
-   * the supplied transaction based on the supplied Session's
-   * transaction information.  This method also ensure the
-   * transaction message is attached properly to the Session object
-   *
-   * @param The transaction message to initialize
-   * @param The Session processing this transaction
-   */
-  void initTransaction(message::Transaction &in_command, Session *in_session) const;
-  /** 
-   * Helper method which finalizes data members for the 
-   * supplied transaction's context.
-   *
-   * @param The transaction message to finalize 
-   * @param The Session processing this transaction
-   */
-  void finalizeTransaction(message::Transaction &in_command, Session *in_session) const;
-  /**
-   * Helper method which deletes transaction memory and
-   * unsets Session's transaction and statement messages.
-   */
-  void cleanupTransaction(message::Transaction *in_transaction,
-                          Session *in_session) const;
-  /**
-   * Helper method which initializes a Statement message
-   *
-   * @param The statement to initialize
-   * @param The type of the statement
-   * @param The session processing this statement
-   */
-  void initStatement(message::Statement &statement,
-                     message::Statement::Type in_type,
-                     Session *in_session) const;
-  /**
-   * Helper method which returns an initialized Statement
-   * message for methods doing insertion of data.
-   *
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table object being inserted into
-   */
-  message::Statement &getInsertStatement(Session *in_session,
-                                         Table *in_table) const;
-
-  /**
-   * Helper method which initializes the header message for
-   * insert operations.
-   *
-   * @param[inout] Statement message container to modify
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table being inserted into
-   */
-  void setInsertHeader(message::Statement &statement,
-                       Session *in_session,
-                       Table *in_table) const;
-  /**
-   * Helper method which returns an initialized Statement
-   * message for methods doing updates of data.
-   *
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table object being updated
-   * @param[in] Pointer to the old data in the record
-   * @param[in] Pointer to the new data in the record
-   */
-  message::Statement &getUpdateStatement(Session *in_session,
-                                         Table *in_table,
-                                         const unsigned char *old_record, 
-                                         const unsigned char *new_record) const;
-  /**
-   * Helper method which initializes the header message for
-   * update operations.
-   *
-   * @param[inout] Statement message container to modify
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table being updated
-   * @param[in] Pointer to the old data in the record
-   * @param[in] Pointer to the new data in the record
-   */
-  void setUpdateHeader(message::Statement &statement,
-                       Session *in_session,
-                       Table *in_table,
-                       const unsigned char *old_record, 
-                       const unsigned char *new_record) const;
-  /**
-   * Helper method which returns an initialized Statement
-   * message for methods doing deletion of data.
-   *
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table object being deleted from
-   */
-  message::Statement &getDeleteStatement(Session *in_session,
-                                         Table *in_table) const;
-
-  /**
-   * Helper method which initializes the header message for
-   * insert operations.
-   *
-   * @param[inout] Statement message container to modify
-   * @param[in] Pointer to the Session doing the processing
-   * @param[in] Pointer to the Table being deleted from
-   */
-  void setDeleteHeader(message::Statement &statement,
-                       Session *in_session,
-                       Table *in_table) const;
-  /**
-   * Helper method which pushes a constructed message out
-   * to the registered replicator and applier plugins.
+public:
+  /** 
+   * Helper method which pushes a constructed message out to the registered
+   * replicator and applier plugins.
    *
    * @param Message to push out
    */
-  void push(message::Transaction &to_push);
-public:
+  void pushTransactionMessage(message::Transaction &to_push);
   /**
    * Constructor
    */
@@ -259,127 +148,8 @@
    * @param Pointer to the applier to detach
    */
   void detachApplier(plugin::TransactionApplier *in_applier);
-  /**
-   * Commits a normal transaction (see above) and pushes the
-   * transaction message out to the replicators.
-   *
-   * @param Pointer to the Session committing the transaction
-   */
-  void commitTransaction(Session *in_session);
-  /**
-   * Marks the current active transaction message as being rolled
-   * back and pushes the transaction message out to replicators.
-   *
-   * @param Pointer to the Session committing the transaction
-   */
-  void rollbackTransaction(Session *in_session);
-  /**
-   * Finalizes a Statement message and sets the Session's statement
-   * message to NULL.
-   *
-   * @param The statement to initialize
-   * @param The session processing this statement
-   */
-  void finalizeStatement(message::Statement &statement,
-                         Session *in_session) const;
-  /**
-   * Creates a new InsertRecord GPB message and pushes it to
-   * replicators.
-   *
-   * @param Pointer to the Session which has inserted a record
-   * @param Pointer to the Table containing insert information
-   *
-   * Grr, returning "true" here on error because of the cursor
-   * reversed bool return crap...fix that.
-   */
-  bool insertRecord(Session *in_session, Table *in_table);
-  /**
-   * Creates a new UpdateRecord GPB message and pushes it to
-   * replicators.
-   *
-   * @param Pointer to the Session which has updated a record
-   * @param Pointer to the Table containing update information
-   * @param Pointer to the raw bytes representing the old record/row
-   * @param Pointer to the raw bytes representing the new record/row 
-   */
-  void updateRecord(Session *in_session, 
-                    Table *in_table, 
-                    const unsigned char *old_record, 
-                    const unsigned char *new_record);
-  /**
-   * Creates a new DeleteRecord GPB message and pushes it to
-   * replicators.
-   *
-   * @param Pointer to the Session which has deleted a record
-   * @param Pointer to the Table containing delete information
-   */
-  void deleteRecord(Session *in_session, Table *in_table);
-  /**
-   * Creates a CreateSchema Statement GPB message and adds it
-   * to the Session's active Transaction GPB message for pushing
-   * out to the replicator streams.
-   *
-   * @param[in] Pointer to the Session which issued the statement
-   * @param[in] message::Schema message describing new schema
-   */
-  void createSchema(Session *in_session, const message::Schema &schema);
-  /**
-   * Creates a DropSchema Statement GPB message and adds it
-   * to the Session's active Transaction GPB message for pushing
-   * out to the replicator streams.
-   *
-   * @param[in] Pointer to the Session which issued the statement
-   * @param[in] message::Schema message describing new schema
-   */
-  void dropSchema(Session *in_session, const std::string &schema_name);
-  /**
-   * Creates a CreateTable Statement GPB message and adds it
-   * to the Session's active Transaction GPB message for pushing
-   * out to the replicator streams.
-   *
-   * @param[in] Pointer to the Session which issued the statement
-   * @param[in] message::Table message describing new schema
-   */
-  void createTable(Session *in_session, const message::Table &table);
-  /**
-   * Creates a DropTable Statement GPB message and adds it
-   * to the Session's active Transaction GPB message for pushing
-   * out to the replicator streams.
-   *
-   * @param[in] Pointer to the Session which issued the statement
-   * @param[in] The schema of the table being dropped
-   * @param[in] The table name of the table being dropped
-   * @param[in] Did the user specify an IF EXISTS clause?
-   */
-  void dropTable(Session *in_session,
-                     const std::string &schema_name,
-                     const std::string &table_name,
-                     bool if_exists);
-  /**
-   * Creates a TruncateTable Statement GPB message and adds it
-   * to the Session's active Transaction GPB message for pushing
-   * out to the replicator streams.
-   *
-   * @param[in] Pointer to the Session which issued the statement
-   * @param[in] The Table being truncated
-   */
-  void truncateTable(Session *in_session, Table *in_table);
-  /**
-   * Creates a new RawSql GPB message and pushes it to 
-   * replicators.
-   *
-   * @TODO With a real data dictionary, this really shouldn't
-   * be needed.  CREATE TABLE would map to insertRecord call
-   * on the I_S, etc.  Not sure what to do with administrative
-   * commands like CHECK TABLE, though..
-   *
-   * @param Pointer to the Session which issued the statement
-   * @param Query string
-   */
-  void rawStatement(Session *in_session, const std::string &query);
-  /**
-   * Returns the timestamp of the last Transaction which was sent to 
-   * an applier.
+  /** Returns the timestamp of the last Transaction which was sent to an
+   * applier.
    */
   uint64_t getLastAppliedTimestamp() const;
 };

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2010-03-06 20:43:53 +0000
+++ drizzled/sql_parse.cc	2010-03-16 21:30:44 +0000
@@ -21,6 +21,7 @@
 #include <drizzled/error.h>
 #include <drizzled/nested_join.h>
 #include <drizzled/query_id.h>
+#include "drizzled/transaction_services.h"
 #include <drizzled/sql_parse.h>
 #include <drizzled/data_home.h>
 #include <drizzled/sql_base.h>
@@ -40,7 +41,6 @@
 #include "drizzled/probes.h"
 #include "drizzled/session_list.h"
 #include "drizzled/global_charset_info.h"
-#include "drizzled/transaction_services.h"
 
 #include "drizzled/plugin/logging.h"
 #include "drizzled/plugin/query_rewrite.h"

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2010-03-04 18:27:48 +0000
+++ drizzled/sql_table.cc	2010-03-16 21:30:44 +0000
@@ -32,7 +32,7 @@
 #include <drizzled/unireg.h>
 #include <drizzled/item/int.h>
 #include <drizzled/item/empty_string.h>
-#include <drizzled/replication_services.h>
+#include <drizzled/transaction_services.h>
 #include "drizzled/transaction_services.h"
 #include <drizzled/table_proto.h>
 #include <drizzled/plugin/client.h>
@@ -106,15 +106,15 @@
 void write_bin_log(Session *session,
                    char const *query)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
-  replication_services.rawStatement(session, query);
+  TransactionServices &transaction_services= TransactionServices::singleton();
+  transaction_services.rawStatement(session, query);
 }
 
 
 /* Should should be refactored to go away */
 void write_bin_log_drop_table(Session *session, bool if_exists, const char *db_name, const char *table_name)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
+  TransactionServices &transaction_services= TransactionServices::singleton();
   string built_query;
 
   if (if_exists)
@@ -131,7 +131,7 @@
 
   built_query.append(table_name);
   built_query.append("`");
-  replication_services.rawStatement(session, built_query);
+  transaction_services.rawStatement(session, built_query);
 }
 
 /*
@@ -263,8 +263,8 @@
 
     if (error == 0 || (if_exists && foreign_key_error == false))
     {
-      ReplicationServices &replication_services= ReplicationServices::singleton();
-      replication_services.dropTable(session, string(db), string(table->table_name), if_exists);
+      TransactionServices &transaction_services= TransactionServices::singleton();
+      transaction_services.dropTable(session, string(db), string(table->table_name), if_exists);
     }
 
     if (error)
@@ -1455,8 +1455,8 @@
 
   if (not internal_tmp_table && not lex_identified_temp_table)
   {
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    replication_services.createTable(session, table_proto);
+    TransactionServices &transaction_services= TransactionServices::singleton();
+    transaction_services.createTable(session, table_proto);
   }
   error= false;
 unlock_and_end:

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2010-03-10 22:47:03 +0000
+++ drizzled/transaction_services.cc	2010-03-16 21:30:44 +0000
@@ -20,6 +20,31 @@
 
 /**
  * @file Transaction processing code
+ *
+ * @note
+ *
+ * The TransactionServices component takes internal events (for instance the start of a 
+ * transaction, the changing of a record, or the rollback of a transaction) 
+ * and constructs GPB Messages that are passed to the ReplicationServices
+ * component and used during replication.
+ *
+ * The reason for this functionality is to encapsulate all communication
+ * between the kernel and the replicator/applier plugins into GPB Messages.
+ * Instead of the plugin having to understand the (often fluidly changing)
+ * mechanics of the kernel, all the plugin needs to understand is the message
+ * format, and GPB messages provide a nice, clear, and versioned format for 
+ * these messages.
+ *
+ * @see /drizzled/message/transaction.proto
+ *
+ * @todo
+ *
+ * We really should store the raw bytes in the messages, not the
+ * String value of the Field.  But, to do that, the
+ * statement_transform library needs first to be updated
+ * to include the transformation code to convert raw
+ * Drizzle-internal Field byte representation into something
+ * plugins can understand.
  */
 
 #include "config.h"
@@ -33,6 +58,8 @@
 #include "drizzled/replication_services.h"
 #include "drizzled/transaction_services.h"
 #include "drizzled/transaction_context.h"
+#include "drizzled/message/transaction.pb.h"
+#include "drizzled/message/statement_transform.h"
 #include "drizzled/resource_context.h"
 #include "drizzled/lock.h"
 #include "drizzled/item/int.h"
@@ -619,8 +646,7 @@
        * We commit the normal transaction by finalizing the transaction message
        * and propogating the message to all registered replicators.
        */
-      ReplicationServices &replication_services= ReplicationServices::singleton();
-      replication_services.commitTransaction(session);
+      commitTransactionMessage(session);
     }
   }
   return error;
@@ -686,8 +712,7 @@
      * a rollback statement with the corresponding transaction ID
      * to rollback.
      */
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    replication_services.rollbackTransaction(session);
+    rollbackTransactionMessage(session);
 
     if (is_real_trans)
       session->transaction.xid_state.xid.null();
@@ -963,4 +988,715 @@
   return error;
 }
 
+message::Transaction *TransactionServices::getActiveTransactionMessage(Session *in_session)
+{
+  message::Transaction *transaction= in_session->getTransactionMessage();
+
+  if (unlikely(transaction == NULL))
+  {
+    /* 
+     * Allocate and initialize a new transaction message 
+     * for this Session object.  Session is responsible for
+     * deleting transaction message when done with it.
+     */
+    transaction= new (nothrow) message::Transaction();
+    initTransactionMessage(*transaction, in_session);
+    in_session->setTransactionMessage(transaction);
+    return transaction;
+  }
+  else
+    return transaction;
+}
+
+void TransactionServices::initTransactionMessage(message::Transaction &in_transaction,
+                                          Session *in_session)
+{
+  message::TransactionContext *trx= in_transaction.mutable_transaction_context();
+  trx->set_server_id(in_session->getServerId());
+  trx->set_transaction_id(in_session->getQueryId());
+  trx->set_start_timestamp(in_session->getCurrentTimestamp());
+}
+
+void TransactionServices::finalizeTransactionMessage(message::Transaction &in_transaction,
+                                              Session *in_session)
+{
+  message::TransactionContext *trx= in_transaction.mutable_transaction_context();
+  trx->set_end_timestamp(in_session->getCurrentTimestamp());
+}
+
+void TransactionServices::cleanupTransactionMessage(message::Transaction *in_transaction,
+                                             Session *in_session)
+{
+  delete in_transaction;
+  in_session->setStatementMessage(NULL);
+  in_session->setTransactionMessage(NULL);
+}
+
+void TransactionServices::commitTransactionMessage(Session *in_session)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+
+  /* If there is an active statement message, finalize it */
+  message::Statement *statement= in_session->getStatementMessage();
+
+  if (statement != NULL)
+  {
+    finalizeStatementMessage(*statement, in_session);
+  }
+  else
+    return; /* No data modification occurred inside the transaction */
+  
+  message::Transaction* transaction= getActiveTransactionMessage(in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+}
+
+void TransactionServices::initStatementMessage(message::Statement &statement,
+                                        message::Statement::Type in_type,
+                                        Session *in_session)
+{
+  statement.set_type(in_type);
+  statement.set_start_timestamp(in_session->getCurrentTimestamp());
+  /** @TODO Set sql string optionally */
+}
+
+void TransactionServices::finalizeStatementMessage(message::Statement &statement,
+                                            Session *in_session)
+{
+  statement.set_end_timestamp(in_session->getCurrentTimestamp());
+  in_session->setStatementMessage(NULL);
+}
+
+void TransactionServices::rollbackTransactionMessage(Session *in_session)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+
+  /*
+   * OK, so there are two situations that we need to deal with here:
+   *
+   * 1) We receive an instruction to ROLLBACK the current transaction
+   *    and the currently-stored Transaction message is *self-contained*, 
+   *    meaning that no Statement messages in the Transaction message
+   *    contain a message having its segment_id member greater than 1.  If
+   *    no non-segment ID 1 members are found, we can simply clear the
+   *    current Transaction message and remove it from memory.
+   *
+   * 2) If the Transaction message does indeed have a non-end segment, that
+   *    means that a bulk update/delete/insert Transaction message segment
+   *    has previously been sent over the wire to replicators.  In this case, 
+   *    we need to package a Transaction with a Statement message of type
+   *    ROLLBACK to indicate to replicators that previously-transmitted
+   *    messages must be un-applied.
+   */
+  if (unlikely(message::transactionContainsBulkSegment(*transaction)))
+  {
+    /*
+     * Clear the transaction, create a Rollback statement message, 
+     * attach it to the transaction, and push it to replicators.
+     */
+    transaction->Clear();
+    initTransactionMessage(*transaction, in_session);
+
+    message::Statement *statement= transaction->add_statement();
+
+    initStatementMessage(*statement, message::Statement::ROLLBACK, in_session);
+    finalizeStatementMessage(*statement, in_session);
+
+    finalizeTransactionMessage(*transaction, in_session);
+    
+    replication_services.pushTransactionMessage(*transaction);
+  }
+  cleanupTransactionMessage(transaction, in_session);
+}
+
+message::Statement &TransactionServices::getInsertStatement(Session *in_session,
+                                                                 Table *in_table)
+{
+  message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type INSERT.
+   * If it is not, we finalize the current Statement and ensure a new
+   * InsertStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::INSERT)
+  {
+    finalizeStatementMessage(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
+
+  if (statement == NULL)
+  {
+    message::Transaction *transaction= getActiveTransactionMessage(in_session);
+    /* 
+     * Transaction message initialized and set, but no statement created
+     * yet.  We construct one and initialize it, here, then return the
+     * message after attaching the new Statement message pointer to the 
+     * Session for easy retrieval later...
+     */
+    statement= transaction->add_statement();
+    setInsertHeader(*statement, in_session, in_table);
+    in_session->setStatementMessage(statement);
+  }
+  return *statement;
+}
+
+void TransactionServices::setInsertHeader(message::Statement &statement,
+                                          Session *in_session,
+                                          Table *in_table)
+{
+  initStatementMessage(statement, message::Statement::INSERT, in_session);
+
+  /* 
+   * Now we construct the specialized InsertHeader message inside
+   * the generalized message::Statement container...
+   */
+  /* Set up the insert header */
+  message::InsertHeader *header= statement.mutable_insert_header();
+  message::TableMetadata *table_metadata= header->mutable_table_metadata();
+
+  const char *schema_name= in_table->getShare()->db.str;
+  const char *table_name= in_table->getShare()->table_name.str;
+
+  table_metadata->set_schema_name(schema_name);
+  table_metadata->set_table_name(table_name);
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+
+  message::FieldMetadata *field_metadata;
+
+  /* We will read all the table's fields... */
+  in_table->setReadSet();
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    field_metadata= header->add_field_metadata();
+    field_metadata->set_name(current_field->field_name);
+    field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
+  }
+}
+
+bool TransactionServices::insertRecord(Session *in_session, Table *in_table)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return false;
+  /**
+   * We do this check here because we don't want to even create a 
+   * statement if there isn't a primary key on the table...
+   *
+   * @todo
+   *
+   * Multi-column primary keys are handled how exactly?
+   */
+  if (in_table->s->primary_key == MAX_KEY)
+  {
+    my_error(ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE, MYF(0));
+    return true;
+  }
+
+  message::Statement &statement= getInsertStatement(in_session, in_table);
+
+  message::InsertData *data= statement.mutable_insert_data();
+  data->set_segment_id(1);
+  data->set_end_segment(true);
+  message::InsertRecord *record= data->add_record();
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+
+  String *string_value= new (in_session->mem_root) String(TransactionServices::DEFAULT_RECORD_SIZE);
+  string_value->set_charset(system_charset_info);
+
+  /* We will read all the table's fields... */
+  in_table->setReadSet();
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    string_value= current_field->val_str(string_value);
+    record->add_insert_value(string_value->c_ptr(), string_value->length());
+    string_value->free();
+  }
+  return false;
+}
+
+message::Statement &TransactionServices::getUpdateStatement(Session *in_session,
+                                                            Table *in_table,
+                                                            const unsigned char *old_record, 
+                                                            const unsigned char *new_record)
+{
+  message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type UPDATE.
+   * If it is not, we finalize the current Statement and ensure a new
+   * UpdateStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::UPDATE)
+  {
+    finalizeStatementMessage(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
+
+  if (statement == NULL)
+  {
+    message::Transaction *transaction= getActiveTransactionMessage(in_session);
+    /* 
+     * Transaction message initialized and set, but no statement created
+     * yet.  We construct one and initialize it, here, then return the
+     * message after attaching the new Statement message pointer to the 
+     * Session for easy retrieval later...
+     */
+    statement= transaction->add_statement();
+    setUpdateHeader(*statement, in_session, in_table, old_record, new_record);
+    in_session->setStatementMessage(statement);
+  }
+  return *statement;
+}
+
+void TransactionServices::setUpdateHeader(message::Statement &statement,
+                                          Session *in_session,
+                                          Table *in_table,
+                                          const unsigned char *old_record, 
+                                          const unsigned char *new_record)
+{
+  initStatementMessage(statement, message::Statement::UPDATE, in_session);
+
+  /* 
+   * Now we construct the specialized UpdateHeader message inside
+   * the generalized message::Statement container...
+   */
+  /* Set up the update header */
+  message::UpdateHeader *header= statement.mutable_update_header();
+  message::TableMetadata *table_metadata= header->mutable_table_metadata();
+
+  const char *schema_name= in_table->getShare()->db.str;
+  const char *table_name= in_table->getShare()->table_name.str;
+
+  table_metadata->set_schema_name(schema_name);
+  table_metadata->set_table_name(table_name);
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+
+  message::FieldMetadata *field_metadata;
+
+  /* We will read all the table's fields... */
+  in_table->setReadSet();
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    /*
+     * We add the "key field metadata" -- i.e. the fields which is
+     * the primary key for the table.
+     */
+    if (in_table->s->fieldInPrimaryKey(current_field))
+    {
+      field_metadata= header->add_key_field_metadata();
+      field_metadata->set_name(current_field->field_name);
+      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
+    }
+
+    /*
+     * The below really should be moved into the Field API and Record API.  But for now
+     * we do this crazy pointer fiddling to figure out if the current field
+     * has been updated in the supplied record raw byte pointers.
+     */
+    const unsigned char *old_ptr= (const unsigned char *) old_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+    const unsigned char *new_ptr= (const unsigned char *) new_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+
+    uint32_t field_length= current_field->pack_length(); /** @TODO This isn't always correct...check varchar diffs. */
+
+    if (memcmp(old_ptr, new_ptr, field_length) != 0)
+    {
+      /* Field is changed from old to new */
+      field_metadata= header->add_set_field_metadata();
+      field_metadata->set_name(current_field->field_name);
+      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
+    }
+  }
+}
+void TransactionServices::updateRecord(Session *in_session,
+                                       Table *in_table, 
+                                       const unsigned char *old_record, 
+                                       const unsigned char *new_record)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+
+  message::Statement &statement= getUpdateStatement(in_session, in_table, old_record, new_record);
+
+  message::UpdateData *data= statement.mutable_update_data();
+  data->set_segment_id(1);
+  data->set_end_segment(true);
+  message::UpdateRecord *record= data->add_record();
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+  String *string_value= new (in_session->mem_root) String(TransactionServices::DEFAULT_RECORD_SIZE);
+  string_value->set_charset(system_charset_info);
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    /*
+     * Here, we add the SET field values.  We used to do this in the setUpdateHeader() method, 
+     * but then realized that an UPDATE statement could potentially have different values for
+     * the SET field.  For instance, imagine this SQL scenario:
+     *
+     * CREATE TABLE t1 (id INT NOT NULL PRIMARY KEY, count INT NOT NULL);
+     * INSERT INTO t1 (id, counter) VALUES (1,1),(2,2),(3,3);
+     * UPDATE t1 SET counter = counter + 1 WHERE id IN (1,2);
+     *
+     * We will generate two UpdateRecord messages with different set_value byte arrays.
+     *
+     * The below really should be moved into the Field API and Record API.  But for now
+     * we do this crazy pointer fiddling to figure out if the current field
+     * has been updated in the supplied record raw byte pointers.
+     */
+    const unsigned char *old_ptr= (const unsigned char *) old_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+    const unsigned char *new_ptr= (const unsigned char *) new_record + (ptrdiff_t) (current_field->ptr - in_table->record[0]); 
+
+    uint32_t field_length= current_field->pack_length(); /** @TODO This isn't always correct...check varchar diffs. */
+
+    if (memcmp(old_ptr, new_ptr, field_length) != 0)
+    {
+      /* Store the original "read bit" for this field */
+      bool is_read_set= current_field->isReadSet();
+
+      /* We need to mark that we will "read" this field... */
+      in_table->setReadSet(current_field->field_index);
+
+      /* Read the string value of this field's contents */
+      string_value= current_field->val_str(string_value);
+
+      /* 
+       * Reset the read bit after reading field to its original state.  This 
+       * prevents the field from being included in the WHERE clause
+       */
+      current_field->setReadSet(is_read_set);
+
+      record->add_after_value(string_value->c_ptr(), string_value->length());
+      string_value->free();
+    }
+
+    /* 
+     * Add the WHERE clause values now...for now, this means the
+     * primary key field value.  Replication only supports tables
+     * with a primary key.
+     */
+    if (in_table->s->fieldInPrimaryKey(current_field))
+    {
+      /**
+       * To say the below is ugly is an understatement. But it works.
+       * 
+       * @todo Move this crap into a real Record API.
+       */
+      string_value= current_field->val_str(string_value,
+                                           old_record + 
+                                           current_field->offset(const_cast<unsigned char *>(new_record)));
+      record->add_key_value(string_value->c_ptr(), string_value->length());
+      string_value->free();
+    }
+
+  }
+}
+
+message::Statement &TransactionServices::getDeleteStatement(Session *in_session,
+                                                            Table *in_table)
+{
+  message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type DELETE.
+   * If it is not, we finalize the current Statement and ensure a new
+   * DeleteStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::DELETE)
+  {
+    finalizeStatementMessage(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
+
+  if (statement == NULL)
+  {
+    message::Transaction *transaction= getActiveTransactionMessage(in_session);
+    /* 
+     * Transaction message initialized and set, but no statement created
+     * yet.  We construct one and initialize it, here, then return the
+     * message after attaching the new Statement message pointer to the 
+     * Session for easy retrieval later...
+     */
+    statement= transaction->add_statement();
+    setDeleteHeader(*statement, in_session, in_table);
+    in_session->setStatementMessage(statement);
+  }
+  return *statement;
+}
+
+void TransactionServices::setDeleteHeader(message::Statement &statement,
+                                          Session *in_session,
+                                          Table *in_table)
+{
+  initStatementMessage(statement, message::Statement::DELETE, in_session);
+
+  /* 
+   * Now we construct the specialized DeleteHeader message inside
+   * the generalized message::Statement container...
+   */
+  message::DeleteHeader *header= statement.mutable_delete_header();
+  message::TableMetadata *table_metadata= header->mutable_table_metadata();
+
+  const char *schema_name= in_table->getShare()->db.str;
+  const char *table_name= in_table->getShare()->table_name.str;
+
+  table_metadata->set_schema_name(schema_name);
+  table_metadata->set_table_name(table_name);
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+
+  message::FieldMetadata *field_metadata;
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    /* 
+     * Add the WHERE clause values now...for now, this means the
+     * primary key field value.  Replication only supports tables
+     * with a primary key.
+     */
+    if (in_table->s->fieldInPrimaryKey(current_field))
+    {
+      field_metadata= header->add_key_field_metadata();
+      field_metadata->set_name(current_field->field_name);
+      field_metadata->set_type(message::internalFieldTypeToFieldProtoType(current_field->type()));
+    }
+  }
+}
+
+void TransactionServices::deleteRecord(Session *in_session, Table *in_table)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+
+  message::Statement &statement= getDeleteStatement(in_session, in_table);
+
+  message::DeleteData *data= statement.mutable_delete_data();
+  data->set_segment_id(1);
+  data->set_end_segment(true);
+  message::DeleteRecord *record= data->add_record();
+
+  Field *current_field;
+  Field **table_fields= in_table->field;
+  String *string_value= new (in_session->mem_root) String(TransactionServices::DEFAULT_RECORD_SIZE);
+  string_value->set_charset(system_charset_info);
+
+  while ((current_field= *table_fields++) != NULL) 
+  {
+    /* 
+     * Add the WHERE clause values now...for now, this means the
+     * primary key field value.  Replication only supports tables
+     * with a primary key.
+     */
+    if (in_table->s->fieldInPrimaryKey(current_field))
+    {
+      string_value= current_field->val_str(string_value);
+      record->add_key_value(string_value->c_ptr(), string_value->length());
+      /**
+       * @TODO Store optional old record value in the before data member
+       */
+      string_value->free();
+    }
+  }
+}
+
+void TransactionServices::createTable(Session *in_session,
+                                      const message::Table &table)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::CREATE_TABLE, in_session);
+
+  /* 
+   * Construct the specialized CreateTableStatement message and attach
+   * it to the generic Statement message
+   */
+  message::CreateTableStatement *create_table_statement= statement->mutable_create_table_statement();
+  message::Table *new_table_message= create_table_statement->mutable_table();
+  *new_table_message= table;
+
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+
+}
+
+void TransactionServices::createSchema(Session *in_session,
+                                       const message::Schema &schema)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::CREATE_SCHEMA, in_session);
+
+  /* 
+   * Construct the specialized CreateSchemaStatement message and attach
+   * it to the generic Statement message
+   */
+  message::CreateSchemaStatement *create_schema_statement= statement->mutable_create_schema_statement();
+  message::Schema *new_schema_message= create_schema_statement->mutable_schema();
+  *new_schema_message= schema;
+
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+
+}
+
+void TransactionServices::dropSchema(Session *in_session, const string &schema_name)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::DROP_SCHEMA, in_session);
+
+  /* 
+   * Construct the specialized DropSchemaStatement message and attach
+   * it to the generic Statement message
+   */
+  message::DropSchemaStatement *drop_schema_statement= statement->mutable_drop_schema_statement();
+
+  drop_schema_statement->set_schema_name(schema_name);
+
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+}
+
+void TransactionServices::dropTable(Session *in_session,
+                                    const string &schema_name,
+                                    const string &table_name,
+                                    bool if_exists)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::DROP_TABLE, in_session);
+
+  /* 
+   * Construct the specialized DropTableStatement message and attach
+   * it to the generic Statement message
+   */
+  message::DropTableStatement *drop_table_statement= statement->mutable_drop_table_statement();
+
+  drop_table_statement->set_if_exists_clause(if_exists);
+
+  message::TableMetadata *table_metadata= drop_table_statement->mutable_table_metadata();
+
+  table_metadata->set_schema_name(schema_name);
+  table_metadata->set_table_name(table_name);
+
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+}
+
+void TransactionServices::truncateTable(Session *in_session, Table *in_table)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::TRUNCATE_TABLE, in_session);
+
+  /* 
+   * Construct the specialized TruncateTableStatement message and attach
+   * it to the generic Statement message
+   */
+  message::TruncateTableStatement *truncate_statement= statement->mutable_truncate_table_statement();
+  message::TableMetadata *table_metadata= truncate_statement->mutable_table_metadata();
+
+  const char *schema_name= in_table->getShare()->db.str;
+  const char *table_name= in_table->getShare()->table_name.str;
+
+  table_metadata->set_schema_name(schema_name);
+  table_metadata->set_table_name(table_name);
+
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+}
+
+void TransactionServices::rawStatement(Session *in_session, const string &query)
+{
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  if (! replication_services.isActive())
+    return;
+  
+  message::Transaction *transaction= getActiveTransactionMessage(in_session);
+  message::Statement *statement= transaction->add_statement();
+
+  initStatementMessage(*statement, message::Statement::RAW_SQL, in_session);
+  statement->set_sql(query);
+  finalizeStatementMessage(*statement, in_session);
+
+  finalizeTransactionMessage(*transaction, in_session);
+  
+  replication_services.pushTransactionMessage(*transaction);
+
+  cleanupTransactionMessage(transaction, in_session);
+}
+
 } /* namespace drizzled */

=== modified file 'drizzled/transaction_services.h'
--- drizzled/transaction_services.h	2010-03-03 20:21:57 +0000
+++ drizzled/transaction_services.h	2010-03-16 21:30:44 +0000
@@ -25,6 +25,8 @@
 #ifndef DRIZZLED_TRANSACTION_SERVICES_H
 #define DRIZZLED_TRANSACTION_SERVICES_H
 
+#include "drizzled/message/transaction.pb.h"
+
 namespace drizzled
 {
 
@@ -46,6 +48,7 @@
 class TransactionServices
 {
 public:
+  static const size_t DEFAULT_RECORD_SIZE= 100;
   /**
    * Constructor
    */
@@ -60,6 +63,235 @@
     static TransactionServices transaction_services;
     return transaction_services;
   }
+  /**
+   * Method which returns the active Transaction message
+   * for the supplied Session.  If one is not found, a new Transaction
+   * message is allocated, initialized, and returned.
+   *
+   * @param The session processing the transaction
+   */
+  message::Transaction *getActiveTransactionMessage(Session *in_session);
+  /** 
+   * Method which attaches a transaction context
+   * the supplied transaction based on the supplied Session's
+   * transaction information.  This method also ensure the
+   * transaction message is attached properly to the Session object
+   *
+   * @param The transaction message to initialize
+   * @param The Session processing this transaction
+   */
+  void initTransactionMessage(message::Transaction &in_transaction, Session *in_session);
+  /** 
+   * Helper method which finalizes data members for the 
+   * supplied transaction's context.
+   *
+   * @param The transaction message to finalize 
+   * @param The Session processing this transaction
+   */
+  void finalizeTransactionMessage(message::Transaction &in_transaction, Session *in_session);
+  /**
+   * Helper method which deletes transaction memory and
+   * unsets Session's transaction and statement messages.
+   */
+  void cleanupTransactionMessage(message::Transaction *in_transaction,
+                                 Session *in_session);
+  /**
+   * Helper method which initializes a Statement message
+   *
+   * @param The statement to initialize
+   * @param The type of the statement
+   * @param The session processing this statement
+   */
+  void initStatementMessage(message::Statement &statement,
+                            message::Statement::Type in_type,
+                            Session *in_session);
+  /**
+   * Finalizes a Statement message and sets the Session's statement
+   * message to NULL.
+   *
+   * @param The statement to initialize
+   * @param The session processing this statement
+   */
+  void finalizeStatementMessage(message::Statement &statement,
+                                Session *in_session);
+  /** Helper method which returns an initialized Statement message for methods
+   * doing insertion of data.
+   *
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table object being inserted into
+   */
+  message::Statement &getInsertStatement(Session *in_session,
+                                         Table *in_table);
+
+  /**
+   * Helper method which initializes the header message for
+   * insert operations.
+   *
+   * @param[inout] Statement message container to modify
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table being inserted into
+   */
+  void setInsertHeader(message::Statement &statement,
+                       Session *in_session,
+                       Table *in_table);
+  /**
+   * Helper method which returns an initialized Statement
+   * message for methods doing updates of data.
+   *
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table object being updated
+   * @param[in] Pointer to the old data in the record
+   * @param[in] Pointer to the new data in the record
+   */
+  message::Statement &getUpdateStatement(Session *in_session,
+                                         Table *in_table,
+                                         const unsigned char *old_record, 
+                                         const unsigned char *new_record);
+  /**
+   * Helper method which initializes the header message for
+   * update operations.
+   *
+   * @param[inout] Statement message container to modify
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table being updated
+   * @param[in] Pointer to the old data in the record
+   * @param[in] Pointer to the new data in the record
+   */
+  void setUpdateHeader(message::Statement &statement,
+                       Session *in_session,
+                       Table *in_table,
+                       const unsigned char *old_record, 
+                       const unsigned char *new_record);
+  /**
+   * Helper method which returns an initialized Statement
+   * message for methods doing deletion of data.
+   *
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table object being deleted from
+   */
+  message::Statement &getDeleteStatement(Session *in_session,
+                                         Table *in_table);
+
+  /**
+   * Helper method which initializes the header message for
+   * insert operations.
+   *
+   * @param[inout] Statement message container to modify
+   * @param[in] Pointer to the Session doing the processing
+   * @param[in] Pointer to the Table being deleted from
+   */
+  void setDeleteHeader(message::Statement &statement,
+                       Session *in_session,
+                       Table *in_table);
+  /** 
+   * Commits a normal transaction (see above) and pushes the transaction
+   * message out to the replicators.
+   *
+   * @param Pointer to the Session committing the transaction
+   */
+  void commitTransactionMessage(Session *in_session);
+  /** 
+   * Marks the current active transaction message as being rolled back and
+   * pushes the transaction message out to replicators.
+   *
+   * @param Pointer to the Session committing the transaction
+   */
+  void rollbackTransactionMessage(Session *in_session);
+  /**
+   * Creates a new InsertRecord GPB message and pushes it to
+   * replicators.
+   *
+   * @param Pointer to the Session which has inserted a record
+   * @param Pointer to the Table containing insert information
+   *
+   * Grr, returning "true" here on error because of the cursor
+   * reversed bool return crap...fix that.
+   */
+  bool insertRecord(Session *in_session, Table *in_table);
+  /**
+   * Creates a new UpdateRecord GPB message and pushes it to
+   * replicators.
+   *
+   * @param Pointer to the Session which has updated a record
+   * @param Pointer to the Table containing update information
+   * @param Pointer to the raw bytes representing the old record/row
+   * @param Pointer to the raw bytes representing the new record/row 
+   */
+  void updateRecord(Session *in_session, 
+                    Table *in_table, 
+                    const unsigned char *old_record, 
+                    const unsigned char *new_record);
+  /**
+   * Creates a new DeleteRecord GPB message and pushes it to
+   * replicators.
+   *
+   * @param Pointer to the Session which has deleted a record
+   * @param Pointer to the Table containing delete information
+   */
+  void deleteRecord(Session *in_session, Table *in_table);
+  /**
+   * Creates a CreateSchema Statement GPB message and adds it
+   * to the Session's active Transaction GPB message for pushing
+   * out to the replicator streams.
+   *
+   * @param[in] Pointer to the Session which issued the statement
+   * @param[in] message::Schema message describing new schema
+   */
+  void createSchema(Session *in_session, const message::Schema &schema);
+  /**
+   * Creates a DropSchema Statement GPB message and adds it
+   * to the Session's active Transaction GPB message for pushing
+   * out to the replicator streams.
+   *
+   * @param[in] Pointer to the Session which issued the statement
+   * @param[in] message::Schema message describing new schema
+   */
+  void dropSchema(Session *in_session, const std::string &schema_name);
+  /**
+   * Creates a CreateTable Statement GPB message and adds it
+   * to the Session's active Transaction GPB message for pushing
+   * out to the replicator streams.
+   *
+   * @param[in] Pointer to the Session which issued the statement
+   * @param[in] message::Table message describing new schema
+   */
+  void createTable(Session *in_session, const message::Table &table);
+  /**
+   * Creates a DropTable Statement GPB message and adds it
+   * to the Session's active Transaction GPB message for pushing
+   * out to the replicator streams.
+   *
+   * @param[in] Pointer to the Session which issued the statement
+   * @param[in] The schema of the table being dropped
+   * @param[in] The table name of the table being dropped
+   * @param[in] Did the user specify an IF EXISTS clause?
+   */
+  void dropTable(Session *in_session,
+                     const std::string &schema_name,
+                     const std::string &table_name,
+                     bool if_exists);
+  /**
+   * Creates a TruncateTable Statement GPB message and adds it
+   * to the Session's active Transaction GPB message for pushing
+   * out to the replicator streams.
+   *
+   * @param[in] Pointer to the Session which issued the statement
+   * @param[in] The Table being truncated
+   */
+  void truncateTable(Session *in_session, Table *in_table);
+  /**
+   * Creates a new RawSql GPB message and pushes it to 
+   * replicators.
+   *
+   * @TODO With a real data dictionary, this really shouldn't
+   * be needed.  CREATE TABLE would map to insertRecord call
+   * on the I_S, etc.  Not sure what to do with administrative
+   * commands like CHECK TABLE, though..
+   *
+   * @param Pointer to the Session which issued the statement
+   * @param Query string
+   */
+  void rawStatement(Session *in_session, const std::string &query);
   /* transactions: interface to plugin::StorageEngine functions */
   int ha_commit_one_phase(Session *session, bool all);
   int ha_rollback_trans(Session *session, bool all);

=== modified file 'plugin/transaction_log/transaction_log_entry.cc'
--- plugin/transaction_log/transaction_log_entry.cc	2009-12-21 09:40:31 +0000
+++ plugin/transaction_log/transaction_log_entry.cc	2010-03-16 21:30:44 +0000
@@ -32,6 +32,8 @@
 
 #include "transaction_log_entry.h"
 
+#include <drizzled/message/transaction.pb.h>
+
 #include <string>
 #include <map>
 

